#include <esp_now.h>
#include <WiFi.h>
#include "Arduino.h"
#include <ArduinoJson.h>
#include <SoftwareSerial.h>

SoftwareSerial linkSerial(16, 17); // RX, TX
SoftwareSerial linkSerial2(22, 23); // RX, TX

TaskHandle_t Task0;
TaskHandle_t Task1;

uint8_t broadcastAddress1[] = {0x24,0x6F,0x28,0x28,0x99,0x9C}; //--> REPLACE WITH THE MAC Address of your receiver.
uint8_t broadcastAddress2[] = {0x3C,0x61,0x05,0x03,0xC2,0xF4};

String success; //--> Variable to store if sending data was successful

//----------------------------------------Structure example to send data
// Must match with ESP32 paired.

typedef struct struct_message {
  int id;
  int led;
  int pm1;
  int pm2_5;
  int pm10;  
} struct_message;

typedef struct shut_led {
  int id;
  int on_off;
} shut_led;

shut_led front;

int send_Data; // Create a struct_message to send data.

struct_message receive_Data; // Create a struct_message to receive data.

struct_message board1;
struct_message board2;

struct_message boardsStruct[2] = {board1,board2};

void OnDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
  Serial.print("\r\nLast Packet Send Status:\t");
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
  if (status ==0){
    success = "Delivery Success :)";
  }
  else{
    success = "Delivery Fail :(";
  }
  char macStr[18];
  Serial.print("Packet to: ");
  // Copies the sender mac address to a string
  snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x",
           mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
  Serial.print(macStr);
  Serial.println(">>>>>");
}

// Callback when data is received
void OnDataRecv(const uint8_t * mac_addr, const uint8_t *incomingData, int len) {
  Serial.println();
  Serial.println("<<<<< Receive Data:");
  char macStr[18];
  Serial.print("Packet received from: ");
  snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x",
           mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
  Serial.println(macStr);
  memcpy(&receive_Data, incomingData, sizeof(receive_Data));
  Serial.printf("Board ID %u: %u bytes\n", receive_Data.id, len);
  //Update the structures with the new incoming data
  boardsStruct[receive_Data.id-1].id = receive_Data.id;
  boardsStruct[receive_Data.id-1].led = receive_Data.led;
  boardsStruct[receive_Data.id-1].pm1 = receive_Data.pm1;
  boardsStruct[receive_Data.id-1].pm2_5 = receive_Data.pm2_5;
  boardsStruct[receive_Data.id-1].pm10 = receive_Data.pm10;
  Serial.printf("led value: %d \n", boardsStruct[receive_Data.id-1].led);
  Serial.printf("pm1 value: %d \n", boardsStruct[receive_Data.id-1].pm1);
  Serial.printf("pm2.5 value: %d \n", boardsStruct[receive_Data.id-1].pm2_5);
  Serial.printf("pm10 value: %d \n", boardsStruct[receive_Data.id-1].pm10); 
  Serial.println();

  StaticJsonDocument<200> doc;
  doc["id"] = receive_Data.id;
  doc["status"] = boardsStruct[receive_Data.id-1].led;
  doc["pm2_5"] = boardsStruct[receive_Data.id-1].pm2_5;
  serializeJson(doc, linkSerial);
  delay(2000);
}

void loop_T1(void * parameter) {
  for (;;) {
    if (linkSerial2.available()) 
      {
        // Allocate the JSON document
        // This one must be bigger than for the sender because it must store the strings
        StaticJsonDocument<300> doc2;
    
        // Read the JSON document from the "link" serial port
        DeserializationError err = deserializeJson(doc2, linkSerial2);
    
        if (err == DeserializationError::Ok) 
        {
          // Print the values
          // (we must use as<T>() to resolve the ambiguity)
          String x = linkSerial2.readStringUntil('}');
          if ((int)x[1]==49 or (int)x[1]==50) {
            if ((int)x[1]==49) {
              front.id = 1;
              front.on_off = (int)x[x.length()-1]-48;
            }
            else {
              front.id = 2; 
              front.on_off = (int)x[x.length()-1]-48;
            }
          }
          /*
          Serial.print("IDRC = ");
          Serial.println(front.id);
          Serial.print("STATUSRC = ");
          Serial.println(front.on_off);*/
        } 
        else 
        {
          // Print error to the "debug" serial port
          Serial.print("deserializeJson() returned ");
          Serial.println(err.c_str());
      
          // Flush all bytes in the "link" serial port buffer
          while (linkSerial2.available() > 0)
            linkSerial2.read();
        }
      }
      delay(2000);
    }    
}

void setup() {
  Serial.begin(115200);
  linkSerial.begin(4800);
  linkSerial2.begin(4800);
  
  WiFi.mode(WIFI_STA); //--> Set device as a Wi-Fi Station

  //Init ESP-NOW
  if (esp_now_init() != ESP_OK) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }
  
  //Once ESPNow is successfully Init, we will register for Send CB to
  // get the status of Trasnmitted packet
  esp_now_register_send_cb(OnDataSent);
  
  //Register peer
  esp_now_peer_info_t peerInfo;
  peerInfo.channel = 0;  
  peerInfo.encrypt = false;
  // register first peer  
  memcpy(peerInfo.peer_addr, broadcastAddress1, 6);
  if (esp_now_add_peer(&peerInfo) != ESP_OK){
    Serial.println("Failed to add peer1");
    return;
  }
  // register second peer  
  memcpy(peerInfo.peer_addr, broadcastAddress2, 6);
  if (esp_now_add_peer(&peerInfo) != ESP_OK){
    Serial.println("Failed to add peer2");
    return;
  }
 
  esp_now_register_recv_cb(OnDataRecv);



  xTaskCreatePinnedToCore(
      loop_T1, /* Function to implement the task */
      "Task1", /* Name of the task */
      5000, /* Stack size in words */
      NULL, /* Task input parameter */
      0, /* Priority of the task */
      &Task1, /* Task handle. */
      1); /* Core where the task should run */
}

void loop() {
  //Set values to send  
  /*
  Serial.println();
  Serial.print(">>>>> ");
  Serial.println("Send data"); */

  //Send message via ESP-NOW
  /*
  if (linkSerial2.available()) 
  {
    // Allocate the JSON document
    // This one must be bigger than for the sender because it must store the strings
    StaticJsonDocument<300> doc2;

    // Read the JSON document from the "link" serial port
    DeserializationError err = deserializeJson(doc2, linkSerial2);

    if (err == DeserializationError::Ok) 
    {
      // Print the values
      // (we must use as<T>() to resolve the ambiguity)
      Serial.print("IDRC = ");
      int x = doc2["id"];
      Serial.println(x);
      Serial.print("STATUSRC = ");
      Serial.println(doc2["on_off"].as<int>());
    } 
    else 
    {
      // Print error to the "debug" serial port
      Serial.print("deserializeJson() returned ");
      Serial.println(err.c_str());
  
      // Flush all bytes in the "link" serial port buffer
      while (linkSerial2.available() > 0)
        linkSerial2.read();
    }
  }
    Serial.println("|||||||||||||");
    Serial.println(front.id);
    Serial.println(front.on_off);
    Serial.println("|||||||||||||"); */
  send_Data = front.on_off;
  esp_err_t result;
  if (front.id == 1){
    esp_err_t result = esp_now_send(broadcastAddress1, (uint8_t *) &send_Data, sizeof(send_Data));  
  }
  else if (front.id == 2){
    esp_err_t result = esp_now_send(broadcastAddress2, (uint8_t *) &send_Data, sizeof(send_Data));
  }
  /*
  else{
    esp_err_t result = esp_now_send(broadcastAddress1, (uint8_t *) &send_Data, sizeof(send_Data));  
    result = esp_now_send(broadcastAddress2, (uint8_t *) &send_Data, sizeof(send_Data));
  }*/
  
  if (result == ESP_OK) {
    Serial.println("Sent with success");
  }
  else {
    Serial.println("Error sending the data");
  }

  
  delay(4000);
}
