#include <ArduinoJson.h>
#include <SoftwareSerial.h>
#include <WiFi.h>
#include <HTTPClient.h>

// Declare the "link" serial port
// Please see SoftwareSerial library for detail
SoftwareSerial linkSerial(16, 17); // RX, TX
SoftwareSerial linkSerial2(22, 23); // RX, TX

char str[50];

const char* ssid = "OnePlus 7 Pro";
const char* password = "a581e18462ef";

StaticJsonDocument<400> JSONPatch; 
StaticJsonDocument<600> JSONGet;
StaticJsonDocument<400> JSONPost; 

typedef struct shut_led {
  String id;
  int on_off;
} shut_led;

shut_led front;

void _postPM(int id, int pm1, int pm2_5, int pm10) {
  String ID;
  if (WiFi.status() == WL_CONNECTED) {
    HTTPClient http;
    
    if (id == 1) {
      http.begin("https://lighting-hole.herokuapp.com/pm/623d76eaa17b8b57451cdd46");
      ID = "623d76eaa17b8b57451cdd46";
    }
    else if (id == 2){
      http.begin("https://lighting-hole.herokuapp.com/pm/623d7392ba85e001a01522d3");
      ID = "623d7392ba85e001a01522d3";
    }
    
    http.addHeader("Content-Type", "application/json");
    JSONPost["pm1"] = pm1;
    JSONPost["pm10"] = pm10;
    JSONPost["pm25"] = pm2_5;
    serializeJson(JSONPost, str);
    int httpCode = http.POST(str);

    if (httpCode == HTTP_CODE_OK) {
      String payload = http.getString();
      Serial.println(httpCode);  
    }
    else {
      Serial.println(httpCode);
      Serial.println("Post Failed");
    }
  }
}

void _patchLight(int id, int led_state) {
  String ID;
  if (WiFi.status() == WL_CONNECTED) {
    HTTPClient http;
    
    if (id == 1) {
      http.begin("https://lighting-hole.herokuapp.com/light/623d76eaa17b8b57451cdd46");
      ID = "623d76eaa17b8b57451cdd46";
    }
    else if (id == 2){
      http.begin("https://lighting-hole.herokuapp.com/light/623d7392ba85e001a01522d3");
      ID = "623d7392ba85e001a01522d3";
    }
    
    http.addHeader("Content-Type", "application/json");
    JSONPatch["status"] = led_state;
    serializeJson(JSONPatch, str);
    int httpCode = http.PATCH(str);

    if (httpCode == HTTP_CODE_OK) {
      String payload = http.getString();
      Serial.println(httpCode);  
    }
    else {
      Serial.println("Patch Failed");
    }
  }
}

void _getLight(int id) {
  if (WiFi.status() == WL_CONNECTED) {
    String ID;
    HTTPClient http;
    
    if (id == 1) {
      http.begin("https://lighting-hole.herokuapp.com/light/623d76eaa17b8b57451cdd46");
      ID = "623d76eaa17b8b57451cdd46";
    }
    else if (id == 2){
      http.begin("https://lighting-hole.herokuapp.com/light/623d7392ba85e001a01522d3");
      ID = "623d7392ba85e001a01522d3";
    }
    
    int httpCode = http.GET();

    if (httpCode == HTTP_CODE_OK) {
      String payload = http.getString();
      DeserializationError err = deserializeJson(JSONGet, payload);
      if (err == DeserializationError::Ok) {
        front.id = (int)JSONGet["_id"];
        front.on_off = (int)JSONGet["status"];
      }
      /*
      Serial.println("+++++++++++++++++++++++++");
      Serial.println(httpCode);
      Serial.print("ID = ");
      Serial.println(id); 
      Serial.print("STATUS = ");
      Serial.println(front.on_off);
      Serial.println("+++++++++++++++++++++++++"); */
      // Create the JSON document
      
      StaticJsonDocument<200> docS;
      docS["id"] = id;
      docS["on_off"] = front.on_off;
     
      // Send the JSON document over the "link" serial port
      serializeJson(docS, linkSerial2);
      
      Serial.println("Sent!");
    }
    else {
      Serial.println("Get Failed");
    }
  }
}


void setup() {
  // Initialize "debug" serial port
  // The data rate must be much higher than the "link" serial port
  Serial.begin(115200);
  while (!Serial) continue;

  WiFi.disconnect();
  WiFi.begin(ssid, password);
  Serial.println("Connecting");
  while(WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected to WiFi network with IP Address: ");  
  // Initialize the "link" serial port
  // Use the lowest possible data rate to reduce error ratio
  linkSerial.begin(4800);
  //linkSerial2.begin(4800);
}
 
void loop() {
  // Check if the other Arduino is transmitting
  int pid;
  int light_status;
  int pm;
  
  if (linkSerial.available()) 
  {
    // Allocate the JSON document
    // This one must be bigger than for the sender because it must store the strings
    StaticJsonDocument<300> doc1;

    // Read the JSON document from the "link" serial port
    DeserializationError err = deserializeJson(doc1, linkSerial);

    if (err == DeserializationError::Ok) 
    {
      // Print the values
      // (we must use as<T>() to resolve the ambiguity)
      pid = doc1["id"].as<int>();
      light_status = doc1["status"].as<int>();
      pm = doc1["pm2_5"].as<int>();
      Serial.println("======================================");
      _postPM(pid, 1, pm, 5);
      _patchLight(pid, light_status);
      //_getLight(pid);
      Serial.print("id = ");
      Serial.println(pid);
      Serial.print("status = ");
      Serial.println(light_status);
      Serial.print("pm2_5 = ");
      Serial.println(pm);
      Serial.println("======================================");
      
      
      
    } 
    else 
    {
      // Print error to the "debug" serial port
      Serial.print("deserializeJson() returned ");
      Serial.println(err.c_str());
  
      // Flush all bytes in the "link" serial port buffer
      while (linkSerial.available() > 0)
        linkSerial.read();
    }
  }
}
