#include <HardwareSerial.h>
#include "Arduino.h"
#include "SoftwareSerial.h"
#include <esp_now.h>
#include <WiFi.h>

SoftwareSerial mySerial(18,19); // RX, TX

uint8_t broadcastAddress[] = {0x3C,0x61,0x05,0x03,0xB5,0xEC};

int receive_data;
int send_led;
int send_pm;

String success;

typedef struct struct_message {
  int id;
  int led;
  int pm1;
  int pm2_5;
  int pm10;
} struct_message;

TaskHandle_t Task0;
TaskHandle_t Task1;

#define RXD2 16
#define TXD2 17

const int main_led = 23;
const int stop_led = 22;
const int ldr = 36;
int main_state = 0;
int stop_state = 0;
int val;

int receive_Data; // Create a struct_message to receive data.

struct_message send_Data; // Create a struct_message to send data.

void OnDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
  Serial.print("\r\nLast Packet Send Status:\t");
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
  if (status ==0){
    success = "Delivery Success :)";
  }
  else{
    success = "Delivery Fail :(";
  }
  char macStr[18];
  Serial.print("Packet from: ");
  // Copies the sender mac address to a string
  snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x",
           mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
  Serial.print(macStr);
  Serial.println(">>>>>");
}

// Callback when data is received
void OnDataRecv(const uint8_t * mac_addr, const uint8_t *incomingData, int len) {
  Serial.println();
  Serial.println("<<<<< Receive Data:");
  char macStr[18];
  Serial.print("Packet received from: ");
  snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x",

           mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
  Serial.println(macStr);
  
  memcpy(&receive_Data, incomingData, sizeof(receive_Data));
  Serial.println("Receive Data: ");
  Serial.println(receive_Data);
  Serial.println("<<<<<");
}

void loop_T0(void * parameter) {
  for (;;) {
    //Serial.println(analogRead(ldr));
    if (analogRead(ldr) >= 1000) {
      main_state = 1;
      digitalWrite(main_led, HIGH);
    }
    else {
      main_state = 0;
      digitalWrite(main_led, LOW);
    }
  }
}

void loop_T1(void * parameter) {
  for (;;) {
      if (mySerial.read() == 1) {
        delay(16000);
        digitalWrite(stop_led, HIGH);
        delay(22000);
        digitalWrite(stop_led, LOW);
        delay(10000);
      }
      digitalWrite(stop_led, LOW);
      delay(10);
    }    
}

void setup() {
  // put your setup code here, to run once:
  pinMode(main_led, OUTPUT);
  pinMode(stop_led, OUTPUT);
  pinMode(ldr, INPUT);

  WiFi.mode(WIFI_STA);
  if (esp_now_init() != ESP_OK) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }
  esp_now_register_send_cb(OnDataSent);
  //Register peer
  esp_now_peer_info_t peerInfo;
  memcpy(peerInfo.peer_addr, broadcastAddress, 6);
  peerInfo.channel = 0;  
  peerInfo.encrypt = false;

  if (esp_now_add_peer(&peerInfo) != ESP_OK){
    Serial.println("Failed to add peer");
    return;
  }
  
  esp_now_register_recv_cb(OnDataRecv);

  mySerial.begin(9600);
  Serial.begin(115200);
  Serial2.begin(9600, SERIAL_8N1, RXD2, TXD2);

  xTaskCreatePinnedToCore(
      loop_T0, /* Function to implement the task */
      "Task0", /* Name of the task */
      1000, /* Stack size in words */
      NULL, /* Task input parameter */
      0, /* Priority of the task */
      &Task0, /* Task handle. */
      0); /* Core where the task should run */

  xTaskCreatePinnedToCore(
      loop_T1, /* Function to implement the task */
      "Task1", /* Name of the task */
      1000, /* Stack size in words */
      NULL, /* Task input parameter */
      0, /* Priority of the task */
      &Task1, /* Task handle. */
      1); /* Core where the task should run */
}
String incoming;
void loop() {
  // put your main code here, to run repeatedly:
  send_Data.id = 1;
  send_Data.led = main_state;
  send_Data.pm1 = 0;  
  
  val = Serial2.read();
  if (val >= 0) {
    send_Data.pm2_5 = val;
  }
  else {
    send_Data.pm2_5 = 0;
  }
  
  send_Data.pm10 = 5; 
  esp_err_t result = esp_now_send(broadcastAddress, (uint8_t *) &send_Data, sizeof(send_Data));
  Serial.println(send_Data.pm2_5); 
  if (result == ESP_OK) {
    Serial.println("Sent with success");
  }
  else {
    Serial.println("Error sending the data");
  }
  delay(2000);
}
